/* SPDX-License-Identifier: BSD-3-Clause */
/*
 * Authors: Wei Chen <wei.chen@arm.com>
 *
 * Copyright (c) 2018, Arm Ltd., All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __PLAT_CMN_IRQ_H__
#define __PLAT_CMN_IRQ_H__

#include <uk/plat/irq.h>

#if defined(__X86_64__)
#include <x86/irq.h>
#elif defined(__ARM_64__)
#include <arm/irq.h>
#else
#error "Add irq.h for current architecture."
#endif

/* define IRQ trigger types */
enum uk_irq_trigger {
	UK_IRQ_TRIGGER_NONE = 0,
	UK_IRQ_TRIGGER_EDGE = 1,
	UK_IRQ_TRIGGER_LEVEL = 2,
	UK_IRQ_TRIGGER_MAX
};

/* define IRQ trigger polarities */
enum uk_irq_polarity {
	UK_IRQ_POLARITY_NONE = 0,
	UK_IRQ_POLARITY_HIGH = 1,
	UK_IRQ_POLARITY_LOW = 2,
	UK_IRQ_POLARITY_MAX
};

#endif /* __PLAT_CMN_IRQ_H__ */
